﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Collections.Generic;
using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Operators;
using HeuristicLab.Optimization;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Analysis.AlgorithmBehavior {

  /// <summary>
  /// An operator that analyzes wildcards.
  /// </summary>
  [Item("WildcardsCreator", "An operator that builds wildcards.")]
  [StorableClass]
  public sealed class WildcardsCreator : SingleSuccessorOperator, IAnalyzer {
    private const string ResultsParameterName = "Results";
    private const string SchemataResultName = "Schemata";
    private const string WildcardsResultName = "Wildcards";

    #region IAnalyzer Members
    public bool EnabledByDefault {
      get { return true; }
    }
    #endregion

    #region Parameter properties
    public ILookupParameter<ResultCollection> ResultsParameter {
      get { return (ILookupParameter<ResultCollection>)Parameters[ResultsParameterName]; }
    }
    #endregion

    #region Properties
    public ResultCollection Results {
      get { return ResultsParameter.ActualValue; }
    }
    #endregion

    [StorableConstructor]
    private WildcardsCreator(bool deserializing) : base(deserializing) { }
    private WildcardsCreator(WildcardsCreator original, Cloner cloner) : base(original, cloner) { }
    public WildcardsCreator()
      : base() {
      Parameters.Add(new LookupParameter<ResultCollection>(ResultsParameterName, "The results collection where the analysis values should be stored."));
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new WildcardsCreator(this, cloner);
    }

    public override IOperation Apply() {
      var schemata = (ItemArray<IntArray>)Results[SchemataResultName].Value;
      var wildcards = new List<IntArray>();
      foreach (var schema in schemata) {
        wildcards.AddRange(AlgorithmBehaviorHelpers.GenerateWildcardSchemata(schema));
      }

      Results.Add(new Result(WildcardsResultName, new ItemArray<IntArray>(wildcards)));
      return base.Apply();
    }
  }
}
